/* File       : debug.h
   Author     : Jorgen Bosman
   Last update: 02/06/1996
   Description: This library uses alternative functions for most of 
   		the standard C functions. These functions check their arguments,
   		check for possible memory faults, check for opened files, etc.
   		At the end of your program, it can display memory leaks and
   		files that are still open.
*/   		

#ifndef _DEBUG_H_
#define _DEBUG_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "dbgfunc.h"

extern char db_verbose;
extern char db_debug_mode;
/* Possible values: 
     0: Do all primary checking
     1: Do all primary checking + check return status and 
     	display error message (default)
*/     	
extern char db_postmortemdump;
/* if >0 (default) then displays a post mortem dump */

/* Use some of these defines if your compiler complains about them */
/* #define __FILE__ "" */
/* #define __FUNCTION__ "" */
/* #define __LINE__ 0 */

/* Don't forget to set _DEBUG_ before you include this file!! 
   Possible values: 
     0 : disables all checking, use original functions
     1 : do all primary checking
*/

void DBInit(char *program_name,char *logfile,int limitlog,void (*SigHand)(int));
/* You can use this function at the start of your program. This function
   Installs some signal handlers, marks the name of the program and creates
   a logfile if necessary. When the program stops, it prints out memory
   leaks, all open files, directories, sockets and pipes.
   Arguments:
     program_name: the name of your program
     logfile: the name of the logfile. If you don't want a logfile,
     	      specify NULL. If the logfile already exists, the new log will be
     	      appended to the old log.
     limitlog: if >0 then limits the size of the logfile to limitlog lines
     SigHand: This function is called when a deadly signal is received with
     	      as argument the received signal
*/

void FinalCheck(void);
/* This function can be called when your program exits. It's not
   necessary to do this when you used DBInit(). It prints out
   all allocated memory and open files. */
   
#define Log	_db_setfileline(__FILE__,__FUNCTION__,__LINE__),_db_log
/* Logs str to the logfile. Logging is preceeded with the current date and time */

#define SP(s)	_db_startprocedure(__FILE__,__FUNCTION__,__LINE__,s)
/* Marks the start of a procedure/function s */

#define EP	_db_endprocedure(__FILE__,__FUNCTION__,__LINE__)
/* Marks the end of a procedure/function */

#define MP	_db_setfileline(__FILE__,__FUNCTION__,__LINE__)
/* Marks a last known position */

#if _DEBUG_==1

#define DEBUG(f) if (db_verbose) {f}
/* Use this to execute some code only when verbose is on */

/* The following functions are replacements for the default functions.
   They check for null pointer assignments and other anomalies. */
   
#define db_malloc(size)		_db_malloc(size,__FILE__,__FUNCTION__,__LINE__)
#define db_free(ptr) 		_db_free(ptr,__FILE__,__FUNCTION__,__LINE__)
#define db_realloc(ptr,size) 	_db_realloc(ptr,size,__FILE__,__FUNCTION__,__LINE__)
#define db_calloc(size,n) 	_db_calloc(size,n,__FILE__,__FUNCTION__,__LINE__)
#define db_strcpy(str1,str2) 	_db_strcpy(str1,str2,__FILE__,__FUNCTION__,__LINE__)
#define db_strncpy(str1,str2,n)	_db_strncpy(str1,str2,n,__FILE__,__FUNCTION__,__LINE__)
#define db_strcmp(str1,str2) 	_db_strcmp(str1,str2,__FILE__,__FUNCTION__,__LINE__)
#define db_strdup(str1) 	_db_strdup(str1,__FILE__,__FUNCTION__,__LINE__)
#define db_memchr(ptr,chr,n)	_db_memchr(ptr,chr,n,__FILE__,__FUNCTION__,__LINE__)
#define db_memcmp(ptr1,ptr2,n)	_db_memcmp(ptr1,ptr2,n,__FILE__,__FUNCTION__,__LINE__)
#define db_memcpy(ptr1,ptr2,n)	_db_memcpy(ptr1,ptr2,n,__FILE__,__FUNCTION__,__LINE__)
#define db_memmove(ptr1,ptr2,n)	_db_memmove(ptr1,ptr2,n,__FILE__,__FUNCTION__,__LINE__)
#define db_memset(ptr,chr,n)	_db_memset(ptr,chr,n,__FILE__,__FUNCTION__,__LINE__)
#define db_strcat(ptr1,ptr2)	_db_strcat(ptr1,ptr2,__FILE__,__FUNCTION__,__LINE__)
#define db_strchr(ptr,chr)	_db_strchr(ptr,chr,__FILE__,__FUNCTION__,__LINE__)
#define db_strcspn(ptr1,ptr2)	_db_strcspn(ptr1,ptr2,__FILE__,__FUNCTION__,__LINE__)
#define db_strlen(ptr)		_db_strlen(ptr,__FILE__,__FUNCTION__,__LINE__)
#define db_strncat(ptr,ptr2,n)	_db_strncat(ptr,ptr2,n,__FILE__,__FUNCTION__,__LINE__)
#define db_strncmp(ptr,ptr2,n)	_db_strncmp(ptr,ptr2,n,__FILE__,__FUNCTION__,__LINE__)
#define db_strpbrk(ptr,ptr2)	_db_strpbrk(ptr,ptr2,__FILE__,__FUNCTION__,__LINE__)
#define db_strrchr(ptr,chr)	_db_strrchr(ptr,chr,__FILE__,__FUNCTION__,__LINE__)
#define db_strspn(ptr,ptr2)	_db_strspn(ptr,ptr2,__FILE__,__FUNCTION__,__LINE__)
#define db_strstr(ptr,ptr2)	_db_strstr(ptr,ptr2,__FILE__,__FUNCTION__,__LINE__)
#define db_strtok(ptr,str2)	_db_strtok(ptr,ptr2,__FILE__,__FUNCTION__,__LINE__)
#define db_fprintf		_db_setfileline(__FILE__,__FUNCTION__,__LINE__),_db_fprintf
#define db_sprintf		_db_setfileline(__FILE__,__FUNCTION__,__LINE__),_db_sprintf

#define db_clearerr(file)	_db_clearerr(file,__FILE__,__FUNCTION__,__LINE__)
#define db_close(file)		_db_close(file,__FILE__,__FUNCTION__,__LINE__)
#define db_creat(file,mode)	_db_creat(file,mode,__FILE__,__FUNCTION__,__LINE__)
#define db_fclose(file)		_db_fclose(file,__FILE__,__FUNCTION__,__LINE__)
#define db_feof(file)		_db_feof(file,__FILE__,__FUNCTION__,__LINE__)
#define db_ferror(file)		_db_ferror(file,__FILE__,__FUNCTION__,__LINE__)
#define db_fflush(file)		_db_fflush(file,__FILE__,__FUNCTION__,__LINE__)
#define db_fgetc(file)		_db_fgetc(file,__FILE__,__FUNCTION__,__LINE__)
#define db_fgets(str,num,file)	_db_fgets(str,num,file,__FILE__,__FUNCTION__,__LINE__)
#define db_fopen(file,mode)	_db_fopen(file,mode,__FILE__,__FUNCTION__,__LINE__)
#define db_fputc(chr,file)	_db_fputc(chr,file,__FILE__,__FUNCTION__,__LINE__)
#define db_fputs(str,file)	_db_fputs(str,file,__FILE__,__FUNCTION__,__LINE__)
#define db_fread(b,s,n,file)	_db_fread(b,s,n,file,__FILE__,__FUNCTION__,__LINE__)
#define db_freopen(n,m,file)	_db_freopen(n,m,file,__FILE__,__FUNCTION__,__LINE__)
#define db_fseek(file,off,or)	_db_fseek(file,off,or,__FILE__,__FUNCTION__,__LINE__)
#define db_ftell(file)		_db_ftell(file,__FILE__,__FUNCTION__,__LINE__)
#define db_fwrite(b,s,n,file)	_db_fwrite(b,s,n,file,__FILE__,__FUNCTION__,__LINE__)
#define db_getc(file)		_db_getc(file,__FILE__,__FUNCTION__,__LINE__)
#define db_lseek(file,off,or)	_db_lseek(file,off,or,__FILE__,__FUNCTION__,__LINE__)
#define db_open(file,mode)	_db_open(file,mode,__FILE__,__FUNCTION__,__LINE__)
#define db_putc(chr,file)	_db_putc(chr,file,__FILE__,__FUNCTION__,__LINE__)
#define db_read(file,buf,cnt)	_db_read(file,buf,cnt,__FILE__,__FUNCTION__,__LINE__)
#define db_remove(file)		_db_remove(file,__FILE__,__FUNCTION__,__LINE__)
#define db_rename(file,tar)	_db_rename(file,tar,__FILE__,__FUNCTION__,__LINE__)
#define db_rewind(file)		_db_rewind(file,__FILE__,__FUNCTION__,__LINE__)
#define db_setbuf(file,buf)	_db_setbuf(file,buf,__FILE__,__FUNCTION__,__LINE__)
#define db_setvbuf(file,b,m,s)	_db_setvbuf(file,b,m,s,__FILE__,__FUNCTION__,__LINE__)
#define db_tmpnam(file)		_db_tmpnam(file,__FILE__,__FUNCTION__,__LINE__)
#define db_tmpfile()		_db_tmpfile(__FILE__,__FUNCTION__,__LINE__)
#define db_ungetc(chr,file)	_db_ungetc(chr,file,__FILE__,__FUNCTION__,__LINE__)
#define db_unlink(file)		_db_unlink(file,__FILE__,__FUNCTION__,__LINE__)
#define db_write(file,b,n)	_db_write(file,b,n,__FILE__,__FUNCTION__,__LINE__)

#define db_pipe(files)		_db_pipe(files,__FILE__,__FUNCTION__,__LINE__)
#define db_dup(file)		_db_dup(file,__FILE__,__FUNCTION__,__LINE__)
#define db_dup2(file,f2)	_db_dup2(file,f2,__FILE__,__FUNCTION__,__LINE__)
#define db_fdopen(file,mode)	_db_fdopen(file,mode,__FILE__,__FUNCTION__,__LINE__)
#define db_fstat(file,st)	_db_fstat(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_lstat(file,st)	_db_lstat(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_stat(file,st)	_db_stat(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_access(file,st)	_db_access(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_opendir(file)	_db_opendir(file,__FILE__,__FUNCTION__,__LINE__)
#define db_readdir(file)	_db_readdir(file,__FILE__,__FUNCTION__,__LINE__)
#define db_closedir(file)	_db_closedir(file,__FILE__,__FUNCTION__,__LINE__)
#define db_chmod(file,st)	_db_chmod(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_fchmod(file,st)	_db_fchmod(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_chown(file,own,grp)	_db_chown(file,own,grp,__FILE__,__FUNCTION__,__LINE__)
#define db_fchown(file,own,grp)	_db_fchown(file,own,grp,__FILE__,__FUNCTION__,__LINE__)
#define db_chdir(file)		_db_chdir(file,__FILE__,__FUNCTION__,__LINE__)
#define db_fchdir(file)		_db_fchdir(file,__FILE__,__FUNCTION__,__LINE__)
#define db_rmdir(file)		_db_rmdir(file,__FILE__,__FUNCTION__,__LINE__)
#define db_truncate(file,st)	_db_truncate(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_ftruncate(file,st)	_db_ftruncate(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_mkdir(file,st)	_db_mkdir(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_link(file,st)	_db_link(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_select(n,r,w,e,t)	_db_select(n,r,w,e,t,__FILE__,__FUNCTION__,__LINE__)
#define db_popen(cmd,st)	_db_popen(cmd,st,__FILE__,__FUNCTION__,__LINE__)
#define db_pclose(file)		_db_pclose(file,__FILE__,__FUNCTION__,__LINE__)
#define db_socket(file,st,re)	_db_socket(file,st,re,__FILE__,__FUNCTION__,__LINE__)
#define db_bind(file,st,re)	_db_bind(file,st,re,__FILE__,__FUNCTION__,__LINE__)
#define db_recv(f,s,n,m)	_db_recv(f,s,n,m,__FILE__,__FUNCTION__,__LINE__)
#define db_send(f,s,n,m)	_db_send(f,s,n,m,__FILE__,__FUNCTION__,__LINE__)
#define db_listen(file,st)	_db_listen(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_shutdown(file,st)	_db_shutdown(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_accept(file,st,re)	_db_accept(file,st,re,__FILE__,__FUNCTION__,__LINE__)
#define db_connect(file,st,re)	_db_connect(file,st,re,__FILE__,__FUNCTION__,__LINE__)
#define db_sendto(f,s,n,m,a,l)	_db_sendto(f,s,n,m,a,l,__FILE__,__FUNCTION__,__LINE__)
#define db_recvfrom(f,s,n,m,a,l) _db_recvfrom(f,s,n,m,a,l,__FILE__,__FUNCTION__,__LINE__)
#define db_getcwd(file,st)	_db_getcwd(file,st,__FILE__,__FUNCTION__,__LINE__)
#define db_scandir(f,s,n,m)	_db_scandir(f,s,n,m,__FILE__,__FUNCTION__,__LINE__)

#else

#define FinalCheck()

#define db_malloc(size)		malloc(size)
#define db_free(ptr) 		free(ptr)
#define db_realloc(ptr,size) 	realloc(ptr,size)
#define db_calloc(size,n) 	calloc(size,n)
#define db_strcpy(str1,str2) 	strcpy(str1,str2)
#define db_strncpy(str1,str2,n)	strncpy(str1,str2,n)
#define db_strcmp(str1,str2) 	strcmp(str1,str2)
#define db_strdup(str1) 	strdup(str1)
#define db_memchr(ptr,chr,n)	memchr(ptr,chr,n)
#define db_memcmp(ptr1,ptr2,n)	memcmp(ptr1,ptr2,n)
#define db_memcpy(ptr1,ptr2,n)	memcpy(ptr1,ptr2,n)
#define db_memmove(ptr1,ptr2,n)	memmove(ptr1,ptr2,n)
#define db_memset(ptr,chr,n)	memset(ptr,chr,n)
#define db_strcat(ptr1,ptr2)	strcat(ptr1,ptr2)
#define db_strchr(ptr,chr)	strchr(ptr,chr)
#define db_strcspn(ptr1,ptr2)	strcspn(ptr1,ptr2)
#define db_strlen(ptr)		strlen(ptr)
#define db_strncat(ptr,ptr2,n)	strncat(ptr,ptr2,n)
#define db_strncmp(ptr,ptr2,n)	strncmp(ptr,ptr2,n)
#define db_strpbrk(ptr,ptr2)	strpbrk(ptr,ptr2)
#define db_strrchr(ptr,chr)	strrchr(ptr,chr)
#define db_strspn(ptr,ptr2)	strspn(ptr,ptr2)
#define db_strstr(ptr,ptr2)	strstr(ptr,ptr2)
#define db_strtok(ptr,str2)	strtok(ptr,ptr2)
#define db_fprintf		fprintf
#define db_sprintf		sprintf

#define db_clearerr(file)	clearerr(file)
#define db_close(file)		close(file)
#define db_creat(file,mode)	creat(file,mode)
#define db_fclose(file)		fclose(file)
#define db_feof(file)		feof(file)
#define db_ferror(file)		ferror(file)
#define db_fflush(file)		fflush(file)
#define db_fgetc(file)		fgetc(file)
#define db_fgets(str,num,file)	fgets(str,num,file)
#define db_fopen(file,mode)	fopen(file,mode)
#define db_fputc(chr,file)	fputc(chr,file)
#define db_fputs(str,file)	fputs(str,file)
#define db_fread(b,s,n,file)	fread(b,s,n,file)
#define db_freopen(n,m,file)	freopen(n,m,file)
#define db_fseek(file,off,or)	fseek(file,off,or)
#define db_ftell(file)		ftell(file)
#define db_fwrite(b,s,n,file)	fwrite(b,s,n,file)
#define db_getc(file)		getc(file)
#define db_lseek(file,off,or)	lseek(file,off,or)
#define db_open(file,mode)	open(file,mode)
#define db_putc(chr,file)	putc(chr,file)
#define db_putchar(chr,file)	putchar(chr,file)
#define db_read(file,buf,cnt)	read(file,buf,cnt)
#define db_remove(file)		remove(file)
#define db_rename(file,tar)	rename(file,tar)
#define db_rewind(file)		rewind(file)
#define db_setbuf(file,buf)	setbuf(file,buf)
#define db_setvbuf(file,b,m,s)	setvbuf(file,b,m,s)
#define db_tmpnam(file)		tmpnam(file)
#define db_tmpfile()		tmpfile()
#define db_ungetc(chr,file)	ungetc(chr,file)
#define db_unlink(file)		unlink(file)
#define db_write(file,b,n)	write(file,b,n)

#define db_pipe(files)		pipe(files)
#define db_dup(file)		dup(file)
#define db_dup2(file,f2)	dup2(file,f2)
#define db_fdopen(file,mode)	fdopen(file,mode)
#define db_fstat(file,st)	fstat(file,st)
#define db_lstat(file,st)	lstat(file,st)
#define db_stat(file,st)	stat(file,st)
#define db_access(file,st)	access(file,st)
#define db_opendir(file)	opendir(file)
#define db_readdir(file)	readdir(file)
#define db_closedir(file)	closedir(file)
#define db_chmod(file,st)	chmod(file,st)
#define db_fchmod(file,st)	fchmod(file,st)
#define db_chown(file,own,grp)	chown(file,own,grp)
#define db_fchown(file,own,grp)	fchown(file,own,grp)
#define db_chdir(file)		chdir(file)
#define db_fchdir(file)		fchdir(file)
#define db_rmdir(file)		rmdir(file)
#define db_truncate(file,st)	truncate(file,st)
#define db_ftruncate(file,st)	ftruncate(file,st)
#define db_mkdir(file,st)	mkdir(file,st)
#define db_link(file,st)	link(file,st)
#define db_select(n,r,w,e,t)	select(n,r,w,e,t)
#define db_popen(cmd,st)	popen(cmd,st)
#define db_pclose(file)		pclose(file)
#define db_socket(file,st,re)	socket(file,st,re)
#define db_bind(file,st,re)	bind(file,st,re)
#define db_recv(f,s,n,m)	recv(f,s,n,m)
#define db_send(f,s,n,m)	send(f,s,n,m)
#define db_listen(file,st)	listen(file,st)
#define db_shutdown(file,st)	shutdown(file,st)
#define db_accept(file,st,re)	accept(file,st,re)
#define db_connect(file,st,re)	connect(file,st,re)
#define db_sendto(f,s,n,m,a,l)	sendto(f,s,n,m,a,l)
#define db_recvfrom(f,s,n,m,a,l) recvfrom(f,s,n,m,a,l)
#define db_getcwd(file,st)	getcwd(file,st)
#define db_scandir(f,s,n,m)	scandir(f,s,n,m)

#define DEBUG(f)


#endif

#endif

